=begin
  version - version information of Ruby-Locale

  Copyright (C) 2008 Masao Mutoh
  Copyright (C) 2013-2015 Kouhei Sutou <kou@clear-code.com>

  You may redistribute it and/or modify it under the same
  license terms as Ruby.
=end
module Locale
  VERSION = "2.1.2"
end

